@@ -1,21 +1,47 @@
-"""A collection of functions designed to help I/O with ascii files."""
+"""A collection of functions designed to help I/O with ascii files.
+
+"""
+from __future__ import division, absolute_import, print_function
+
 __docformat__ = "restructuredtext en"
 
 import sys
 import numpy as np
 import numpy.core.numeric as nx
-from __builtin__ import bool, int, long, float, complex, object, unicode, str
-
-from numpy.compat import asbytes, bytes, asbytes_nested
+from numpy.compat import asbytes, asunicode, bytes, asbytes_nested, basestring
 
 if sys.version_info[0] >= 3:
-    def _bytes_to_complex(s):
-        return complex(s.decode('ascii'))
-    def _bytes_to_name(s):
-        return s.decode('ascii')
+    from builtins import bool, int, float, complex, object, str
+    unicode = str
 else:
-    _bytes_to_complex = complex
-    _bytes_to_name = str
+    from __builtin__ import bool, int, float, complex, object, unicode, str
+
+
+def _decode_line(line, encoding=None):
+    """Decode bytes from binary input streams.
+
+    Defaults to decoding from 'latin1'. That differs from the behavior of
+    np.compat.asunicode that decodes from 'ascii'.
+
+    Parameters
+    ----------
+    line : str or bytes
+         Line to be decoded.
+
+    Returns
+    -------
+    decoded_line : unicode
+         Unicode in Python 2, a str (unicode) in Python 3.
+
+    """
+    if type(line) is bytes:
+        if encoding is None:
+            line = line.decode('latin1')
+        else:
+            line = line.decode(encoding)
+
+    return line
+
 
 def _is_string_like(obj):
     """
@@ -27,12 +53,13 @@ def _is_string_like(obj):
         return False
     return True
 
+
 def _is_bytes_like(obj):
     """
     Check whether obj behaves like a bytes object.
     """
     try:
-        obj + asbytes('')
+        obj + b''
     except (TypeError, ValueError):
         return False
     return True
@@ -42,7 +69,7 @@ def _to_filehandle(fname, flag='r', return_opened=False):
     """
     Returns the filehandle corresponding to a string or a file.
     If the string ends in '.gz', the file is automatically unzipped.
-    
+
     Parameters
     ----------
     fname : string, filehandle
@@ -83,7 +110,8 @@ def has_nested_fields(ndtype):
 
     Raises
     ------
-    AttributeError : If `ndtype` does not have a `names` attribute.
+    AttributeError
+        If `ndtype` does not have a `names` attribute.
 
     Examples
     --------
@@ -109,8 +137,9 @@ def flatten_dtype(ndtype, flatten_base=False):
     ----------
     ndtype : dtype
         The datatype to collapse
-    flatten_base : {False, True}, optional
-        Whether to transform a field with a shape into several fields or not.
+    flatten_base : bool, optional
+       If True, transform a field with a shape into several fields. Default is
+       False.
 
     Examples
     --------
@@ -138,11 +167,7 @@ def flatten_dtype(ndtype, flatten_base=False):
         return types
 
 
-
-
-
-
-class LineSplitter:
+class LineSplitter(object):
     """
     Object to split a string at a given delimiter or at given places.
 
@@ -151,7 +176,7 @@ class LineSplitter:
     delimiter : str, int, or sequence of ints, optional
         If a string, character used to delimit consecutive fields.
         If an integer or a sequence of integers, width(s) of each field.
-    comment : str, optional
+    comments : str, optional
         Character used to mark the beginning of a comment. Default is '#'.
     autostrip : bool, optional
         Whether to strip each individual field. Default is True.
@@ -178,12 +203,15 @@ class LineSplitter:
         """
         return lambda input: [_.strip() for _ in method(input)]
     #
-    def __init__(self, delimiter=None, comments=asbytes('#'), autostrip=True):
+
+    def __init__(self, delimiter=None, comments='#', autostrip=True, encoding=None):
+        delimiter = _decode_line(delimiter)
+        comments = _decode_line(comments)
+
         self.comments = comments
+
         # Delimiter is a character
-        if isinstance(delimiter, unicode):
-            delimiter = delimiter.encode('ascii')
-        if (delimiter is None) or _is_bytes_like(delimiter):
+        if (delimiter is None) or isinstance(delimiter, basestring):
             delimiter = delimiter or None
             _handyman = self._delimited_splitter
         # Delimiter is a list of field widths
@@ -193,7 +221,8 @@ class LineSplitter:
             delimiter = [slice(i, j) for (i, j) in zip(idx[:-1], idx[1:])]
         # Delimiter is a single integer
         elif int(delimiter):
-            (_handyman, delimiter) = (self._fixedwidth_splitter, int(delimiter))
+            (_handyman, delimiter) = (
+                    self._fixedwidth_splitter, int(delimiter))
         else:
             (_handyman, delimiter) = (self._delimited_splitter, None)
         self.delimiter = delimiter
@@ -201,70 +230,80 @@ class LineSplitter:
             self._handyman = self.autostrip(_handyman)
         else:
             self._handyman = _handyman
+        self.encoding = encoding
     #
+
     def _delimited_splitter(self, line):
-        line = line.split(self.comments)[0].strip(asbytes(" \r\n"))
+        """Chop off comments, strip, and split at delimiter. """
+        if self.comments is not None:
+            line = line.split(self.comments)[0]
+        line = line.strip(" \r\n")
         if not line:
             return []
         return line.split(self.delimiter)
     #
+
     def _fixedwidth_splitter(self, line):
-        line = line.split(self.comments)[0].strip(asbytes("\r\n"))
+        if self.comments is not None:
+            line = line.split(self.comments)[0]
+        line = line.strip("\r\n")
         if not line:
             return []
         fixed = self.delimiter
         slices = [slice(i, i + fixed) for i in range(0, len(line), fixed)]
         return [line[s] for s in slices]
     #
+
     def _variablewidth_splitter(self, line):
-        line = line.split(self.comments)[0]
+        if self.comments is not None:
+            line = line.split(self.comments)[0]
         if not line:
             return []
         slices = self.delimiter
         return [line[s] for s in slices]
     #
+
     def __call__(self, line):
-        return self._handyman(line)
+        return self._handyman(_decode_line(line, self.encoding))
 
 
-
-class NameValidator:
+class NameValidator(object):
     """
     Object to validate a list of strings to use as field names.
 
     The strings are stripped of any non alphanumeric character, and spaces
-    are replaced by '_'. During instantiation, the user can define a list of
-    names to exclude, as well as a list of invalid characters. Names in the
-    exclusion list are appended a '_' character.
+    are replaced by '_'. During instantiation, the user can define a list
+    of names to exclude, as well as a list of invalid characters. Names in
+    the exclusion list are appended a '_' character.
 
-    Once an instance has been created, it can be called with a list of names,
-    and a list of valid names will be created.
-    The `__call__` method accepts an optional keyword "default" that sets
-    the default name in case of ambiguity. By default this is 'f', so
-    that names will default to `f0`, `f1`, etc.
+    Once an instance has been created, it can be called with a list of
+    names, and a list of valid names will be created.  The `__call__`
+    method accepts an optional keyword "default" that sets the default name
+    in case of ambiguity. By default this is 'f', so that names will
+    default to `f0`, `f1`, etc.
 
     Parameters
     ----------
     excludelist : sequence, optional
-        A list of names to exclude. This list is appended to the default list
-        ['return', 'file', 'print']. Excluded names are appended an underscore:
-        for example, `file` becomes `file_` if supplied.
+        A list of names to exclude. This list is appended to the default
+        list ['return', 'file', 'print']. Excluded names are appended an
+        underscore: for example, `file` becomes `file_` if supplied.
     deletechars : str, optional
         A string combining invalid characters that must be deleted from the
         names.
-    casesensitive : {True, False, 'upper', 'lower'}, optional
+    case_sensitive : {True, False, 'upper', 'lower'}, optional
         * If True, field names are case-sensitive.
         * If False or 'upper', field names are converted to upper case.
         * If 'lower', field names are converted to lower case.
 
         The default value is True.
-    replace_space: '_', optional
+    replace_space : '_', optional
         Character(s) used in replacement of white spaces.
 
     Notes
     -----
-    Calling an instance of `NameValidator` is the same as calling its method
-    `validate`.
+    Calling an instance of `NameValidator` is the same as calling its
+    method `validate`.
 
     Examples
     --------
@@ -281,8 +320,9 @@ class NameValidator:
     """
     #
     defaultexcludelist = ['return', 'file', 'print']
-    defaultdeletechars = set("""~!@#$%^&*()-=+~\|]}[{';: /?.>,<""")
+    defaultdeletechars = set(r"""~!@#$%^&*()-=+~\|]}[{';: /?.>,<""")
     #
+
     def __init__(self, excludelist=None, deletechars=None,
                  case_sensitive=None, replace_space='_'):
         # Process the exclusion list ..
@@ -300,29 +340,30 @@ class NameValidator:
         # Process the case option .....
         if (case_sensitive is None) or (case_sensitive is True):
             self.case_converter = lambda x: x
-        elif (case_sensitive is False) or ('u' in case_sensitive):
+        elif (case_sensitive is False) or case_sensitive.startswith('u'):
             self.case_converter = lambda x: x.upper()
-        elif 'l' in case_sensitive:
+        elif case_sensitive.startswith('l'):
             self.case_converter = lambda x: x.lower()
         else:
-            self.case_converter = lambda x: x
+            msg = 'unrecognized case_sensitive value %s.' % case_sensitive
+            raise ValueError(msg)
         #
         self.replace_space = replace_space
 
     def validate(self, names, defaultfmt="f%i", nbfields=None):
         """
-        Validate a list of strings to use as field names for a structured array.
+        Validate a list of strings as field names for a structured array.
 
         Parameters
         ----------
         names : sequence of str
             Strings to be validated.
         defaultfmt : str, optional
-            Default format string, used if validating a given string reduces its
-            length to zero.
-        nboutput : integer, optional
-            Final number of validated names, used to expand or shrink the initial
-            list of names.
+            Default format string, used if validating a given string
+            reduces its length to zero.
+        nbfields : integer, optional
+            Final number of validated names, used to expand or shrink the
+            initial list of names.
 
         Returns
         -------
@@ -331,8 +372,8 @@ class NameValidator:
 
         Notes
         -----
-        A `NameValidator` instance can be called directly, which is the same as
-        calling `validate`. For examples, see `NameValidator`.
+        A `NameValidator` instance can be called directly, which is the
+        same as calling `validate`. For examples, see `NameValidator`.
 
         """
         # Initial checks ..............
@@ -379,11 +420,11 @@ class NameValidator:
             seen[item] = cnt + 1
         return tuple(validatednames)
     #
+
     def __call__(self, names, defaultfmt="f%i", nbfields=None):
         return self.validate(names, defaultfmt=defaultfmt, nbfields=nbfields)
 
 
-
 def str2bool(value):
     """
     Tries to transform a string supposed to represent a boolean to a boolean.
@@ -412,9 +453,9 @@ def str2bool(value):
 
     """
     value = value.upper()
-    if value == asbytes('TRUE'):
+    if value == 'TRUE':
         return True
-    elif value == asbytes('FALSE'):
+    elif value == 'FALSE':
         return False
     else:
         raise ValueError("Invalid boolean")
@@ -427,6 +468,7 @@ class ConverterError(Exception):
     """
     pass
 
+
 class ConverterLockError(ConverterError):
     """
     Exception raised when an attempt is made to upgrade a locked converter.
@@ -434,6 +476,7 @@ class ConverterLockError(ConverterError):
     """
     pass
 
+
 class ConversionWarning(UserWarning):
     """
     Warning issued when a string converter has a problem.
@@ -447,22 +490,22 @@ class ConversionWarning(UserWarning):
     pass
 
 
-
-class StringConverter:
+class StringConverter(object):
     """
-    Factory class for function transforming a string into another object (int,
-    float).
+    Factory class for function transforming a string into another object
+    (int, float).
 
     After initialization, an instance can be called to transform a string
-    into another object. If the string is recognized as representing a missing
-    value, a default value is returned.
+    into another object. If the string is recognized as representing a
+    missing value, a default value is returned.
 
     Attributes
     ----------
     func : function
         Function used for the conversion.
     default : any
-        Default value to return when the input corresponds to a missing value.
+        Default value to return when the input corresponds to a missing
+        value.
     type : type
         Type of the output.
     _status : int
@@ -479,16 +522,17 @@ class StringConverter:
         If a `dtype`, specifies the input data type, used to define a basic
         function and a default value for missing data. For example, when
         `dtype` is float, the `func` attribute is set to `float` and the
-        default value to `np.nan`.
-        If a function, this function is used to convert a string to another
-        object. In this case, it is recommended to give an associated default
-        value as input.
+        default value to `np.nan`.  If a function, this function is used to
+        convert a string to another object. In this case, it is recommended
+        to give an associated default value as input.
     default : any, optional
-        Value to return by default, that is, when the string to be converted
-        is flagged as missing. If not given, `StringConverter` tries to supply
-        a reasonable default value.
-    missing_values : sequence of str, optional
-        Sequence of strings indicating a missing value.
+        Value to return by default, that is, when the string to be
+        converted is flagged as missing. If not given, `StringConverter`
+        tries to supply a reasonable default value.
+    missing_values : {None, sequence of str}, optional
+        ``None`` or sequence of strings indicating a missing value. If ``None``
+        then missing values are indicated by empty entries. The default is
+        ``None``.
     locked : bool, optional
         Whether the StringConverter should be locked to prevent automatic
         upgrade or not. Default is False.
@@ -496,27 +540,56 @@ class StringConverter:
     """
     #
     _mapper = [(nx.bool_, str2bool, False),
-               (nx.integer, int, -1),
-               (nx.floating, float, nx.nan),
-               (complex, _bytes_to_complex, nx.nan + 0j),
-               (nx.string_, bytes, asbytes('???'))]
+               (nx.integer, int, -1)]
+
+    # On 32-bit systems, we need to make sure that we explicitly include
+    # nx.int64 since ns.integer is nx.int32.
+    if nx.dtype(nx.integer).itemsize < nx.dtype(nx.int64).itemsize:
+        _mapper.append((nx.int64, int, -1))
+
+    _mapper.extend([(nx.floating, float, nx.nan),
+                    (nx.complexfloating, complex, nx.nan + 0j),
+                    (nx.longdouble, nx.longdouble, nx.nan),
+                    (nx.unicode_, asunicode, '???'),
+                    (nx.string_, asbytes, '???')])
+
     (_defaulttype, _defaultfunc, _defaultfill) = zip(*_mapper)
+
+    @classmethod
+    def _getdtype(cls, val):
+        """Returns the dtype of the input variable."""
+        return np.array(val).dtype
     #
+
     @classmethod
     def _getsubdtype(cls, val):
         """Returns the type of the dtype of the input variable."""
         return np.array(val).dtype.type
     #
+    # This is a bit annoying. We want to return the "general" type in most
+    # cases (ie. "string" rather than "S10"), but we want to return the
+    # specific type for datetime64 (ie. "datetime64[us]" rather than
+    # "datetime64").
+
+    @classmethod
+    def _dtypeortype(cls, dtype):
+        """Returns dtype for datetime64 and type of dtype otherwise."""
+        if dtype.type == np.datetime64:
+            return dtype
+        return dtype.type
+    #
+
     @classmethod
     def upgrade_mapper(cls, func, default=None):
         """
-    Upgrade the mapper of a StringConverter by adding a new function and its
-    corresponding default.
-    
-    The input function (or sequence of functions) and its associated default 
-    value (if any) is inserted in penultimate position of the mapper.
-    The corresponding type is estimated from the dtype of the default value.
-    
+    Upgrade the mapper of a StringConverter by adding a new function and
+    its corresponding default.
+
+    The input function (or sequence of functions) and its associated
+    default value (if any) is inserted in penultimate position of the
+    mapper.  The corresponding type is estimated from the dtype of the
+    default value.
+
     Parameters
     ----------
     func : var
@@ -547,13 +620,9 @@ class StringConverter:
             for (fct, dft) in zip(func, default):
                 cls._mapper.insert(-1, (cls._getsubdtype(dft), fct, dft))
     #
+
     def __init__(self, dtype_or_func=None, default=None, missing_values=None,
                  locked=False):
-        # Convert unicode (for Py3)
-        if isinstance(missing_values, unicode):
-            missing_values = asbytes(missing_values)
-        elif isinstance(missing_values, (list, tuple)):
-            missing_values = asbytes_nested(missing_values)
         # Defines a lock for upgrade
         self._locked = bool(locked)
         # No input dtype: minimal initialization
@@ -561,37 +630,50 @@ class StringConverter:
             self.func = str2bool
             self._status = 0
             self.default = default or False
-            ttype = np.bool
+            dtype = np.dtype('bool')
         else:
             # Is the input a np.dtype ?
             try:
                 self.func = None
-                ttype = np.dtype(dtype_or_func).type
+                dtype = np.dtype(dtype_or_func)
             except TypeError:
                 # dtype_or_func must be a function, then
                 if not hasattr(dtype_or_func, '__call__'):
-                    errmsg = "The input argument `dtype` is neither a function"\
-                             " or a dtype (got '%s' instead)"
+                    errmsg = ("The input argument `dtype` is neither a"
+                              " function nor a dtype (got '%s' instead)")
                     raise TypeError(errmsg % type(dtype_or_func))
                 # Set the function
                 self.func = dtype_or_func
-                # If we don't have a default, try to guess it or set it to None
+                # If we don't have a default, try to guess it or set it to
+                # None
                 if default is None:
                     try:
-                        default = self.func(asbytes('0'))
+                        default = self.func('0')
                     except ValueError:
                         default = None
-                ttype = self._getsubdtype(default)
+                dtype = self._getdtype(default)
             # Set the status according to the dtype
             _status = -1
             for (i, (deftype, func, default_def)) in enumerate(self._mapper):
-                if np.issubdtype(ttype, deftype):
+                if np.issubdtype(dtype.type, deftype):
                     _status = i
                     if default is None:
                         self.default = default_def
                     else:
                         self.default = default
                     break
+            # if a converter for the specific dtype is available use that
+            last_func = func
+            for (i, (deftype, func, default_def)) in enumerate(self._mapper):
+                if dtype.type == deftype:
+                    _status = i
+                    last_func = func
+                    if default is None:
+                        self.default = default_def
+                    else:
+                        self.default = default
+                    break
+            func = last_func
             if _status == -1:
                 # We never found a match in the _mapper...
                 _status = 0
@@ -603,34 +685,51 @@ class StringConverter:
             # If the status is 1 (int), change the function to
             # something more robust.
             if self.func == self._mapper[1][1]:
-                if issubclass(ttype, np.uint64):
+                if issubclass(dtype.type, np.uint64):
                     self.func = np.uint64
-                elif issubclass(ttype, np.int64):
+                elif issubclass(dtype.type, np.int64):
                     self.func = np.int64
                 else:
-                    self.func = lambda x : int(float(x))
+                    self.func = lambda x: int(float(x))
         # Store the list of strings corresponding to missing values.
         if missing_values is None:
-            self.missing_values = set([asbytes('')])
+            self.missing_values = set([''])
         else:
-            if isinstance(missing_values, bytes):
-                missing_values = missing_values.split(asbytes(","))
-            self.missing_values = set(list(missing_values) + [asbytes('')])
+            if isinstance(missing_values, basestring):
+                missing_values = missing_values.split(",")
+            self.missing_values = set(list(missing_values) + [''])
         #
         self._callingfunction = self._strict_call
-        self.type = ttype
+        self.type = self._dtypeortype(dtype)
         self._checked = False
         self._initial_default = default
     #
+
     def _loose_call(self, value):
         try:
             return self.func(value)
         except ValueError:
             return self.default
     #
+
     def _strict_call(self, value):
         try:
-            return self.func(value)
+
+            # We check if we can convert the value using the current function
+            new_value = self.func(value)
+
+            # In addition to having to check whether func can convert the
+            # value, we also have to make sure that we don't get overflow
+            # errors for integers.
+            if self.func is int:
+                try:
+                    np.array(value, dtype=self.type)
+                except OverflowError:
+                    raise ValueError
+
+            # We're still here so we can now return the new value
+            return new_value
+
         except ValueError:
             if value.strip() in self.missing_values:
                 if not self._status:
@@ -638,18 +737,20 @@ class StringConverter:
                 return self.default
             raise ValueError("Cannot convert string '%s'" % value)
     #
+
     def __call__(self, value):
         return self._callingfunction(value)
     #
+
     def upgrade(self, value):
         """
-        Try to find the best converter for a given string, and return the result.
+        Find the best converter for a given string, and return the result.
 
         The supplied string `value` is converted by testing different
-        converters in order. First the `func` method of the `StringConverter`
-        instance is tried, if this fails other available converters are tried.
-        The order in which these other converters are tried is determined by the
-        `_status` attribute of the instance.
+        converters in order. First the `func` method of the
+        `StringConverter` instance is tried, if this fails other available
+        converters are tried.  The order in which these other converters
+        are tried is determined by the `_status` attribute of the instance.
 
         Parameters
         ----------
@@ -664,7 +765,7 @@ class StringConverter:
         """
         self._checked = True
         try:
-            self._strict_call(value)
+            return self._strict_call(value)
         except ValueError:
             # Raise an exception if we locked the converter...
             if self._locked:
@@ -684,7 +785,7 @@ class StringConverter:
                 self.default = self._initial_default
             else:
                 self.default = default
-            self.upgrade(value)
+            return self.upgrade(value)
 
     def iterupgrade(self, value):
         self._checked = True
@@ -692,7 +793,8 @@ class StringConverter:
             value = (value,)
         _strict_call = self._strict_call
         try:
-            map(_strict_call, value)
+            for _m in value:
+                _strict_call(_m)
         except ValueError:
             # Raise an exception if we locked the converter...
             if self._locked:
@@ -702,7 +804,9 @@ class StringConverter:
             # Complains if we try to upgrade by the maximum
             _status = self._status
             if _status == _statusmax:
-                raise ConverterError("Could not find a valid conversion function")
+                raise ConverterError(
+                    "Could not find a valid conversion function"
+                    )
             elif _status < _statusmax - 1:
                 _status += 1
             (self.type, self.func, default) = self._mapper[_status]
@@ -714,7 +818,7 @@ class StringConverter:
             self.iterupgrade(value)
 
     def update(self, func, default=None, testing_value=None,
-               missing_values=asbytes(''), locked=False):
+               missing_values='', locked=False):
         """
         Set StringConverter attributes directly.
 
@@ -723,47 +827,52 @@ class StringConverter:
         func : function
             Conversion function.
         default : any, optional
-            Value to return by default, that is, when the string to be converted
-            is flagged as missing. If not given, `StringConverter` tries to supply
-            a reasonable default value.
+            Value to return by default, that is, when the string to be
+            converted is flagged as missing. If not given,
+            `StringConverter` tries to supply a reasonable default value.
         testing_value : str, optional
             A string representing a standard input value of the converter.
-            This string is used to help defining a reasonable default value.
-        missing_values : sequence of str, optional
-            Sequence of strings indicating a missing value.
+            This string is used to help defining a reasonable default
+            value.
+        missing_values : {sequence of str, None}, optional
+            Sequence of strings indicating a missing value. If ``None``, then
+            the existing `missing_values` are cleared. The default is `''`.
         locked : bool, optional
-            Whether the StringConverter should be locked to prevent automatic
-            upgrade or not. Default is False.
+            Whether the StringConverter should be locked to prevent
+            automatic upgrade or not. Default is False.
 
         Notes
         -----
-        `update` takes the same parameters as the constructor of `StringConverter`,
-        except that `func` does not accept a `dtype` whereas `dtype_or_func` in
-        the constructor does.
+        `update` takes the same parameters as the constructor of
+        `StringConverter`, except that `func` does not accept a `dtype`
+        whereas `dtype_or_func` in the constructor does.
 
         """
         self.func = func
         self._locked = locked
+
         # Don't reset the default to None if we can avoid it
         if default is not None:
             self.default = default
-            self.type = self._getsubdtype(default)
+            self.type = self._dtypeortype(self._getdtype(default))
         else:
             try:
-                tester = func(testing_value or asbytes('1'))
+                tester = func(testing_value or '1')
             except (TypeError, ValueError):
                 tester = None
-            self.type = self._getsubdtype(tester)
-        # Add the missing values to the existing set
-        if missing_values is not None:
-            if _is_bytes_like(missing_values):
-                self.missing_values.add(missing_values)
-            elif hasattr(missing_values, '__iter__'):
-                for val in missing_values:
-                    self.missing_values.add(val)
-        else:
-            self.missing_values = []
+            self.type = self._dtypeortype(self._getdtype(tester))
 
+        # Add the missing values to the existing set or clear it.
+        if missing_values is None:
+            # Clear all missing values even though the ctor initializes it to
+            # set(['']) when the argument is None.
+            self.missing_values = set()
+        else:
+            if not np.iterable(missing_values):
+                missing_values = [missing_values]
+            if not all(isinstance(v, basestring) for v in missing_values):
+                raise TypeError("missing_values must be strings or unicode")
+            self.missing_values.update(missing_values)
 
 
 def easy_dtype(ndtype, names=None, defaultfmt="f%i", **validationargs):
@@ -776,17 +885,18 @@ def easy_dtype(ndtype, names=None, defaultfmt="f%i", **validationargs):
     Parameters
     ----------
     ndtype : var
-        Definition of the dtype. Can be any string or dictionary
-        recognized by the `np.dtype` function, or a sequence of types.
+        Definition of the dtype. Can be any string or dictionary recognized
+        by the `np.dtype` function, or a sequence of types.
     names : str or sequence, optional
         Sequence of strings to use as field names for a structured dtype.
-        For convenience, `names` can be a string of a comma-separated list of
-        names.
+        For convenience, `names` can be a string of a comma-separated list
+        of names.
     defaultfmt : str, optional
         Format string used to define missing names, such as ``"f%i"``
         (default) or ``"fields_%02i"``.
     validationargs : optional
-        A series of optional arguments used to initialize a `NameValidator`.
+        A series of optional arguments used to initialize a
+        `NameValidator`.
 
     Examples
     --------
@@ -825,7 +935,7 @@ def easy_dtype(ndtype, names=None, defaultfmt="f%i", **validationargs):
             if nbtypes == 0:
                 formats = tuple([ndtype.type] * len(names))
                 names = validate(names, defaultfmt=defaultfmt)
-                ndtype = np.dtype(zip(names, formats))
+                ndtype = np.dtype(list(zip(names, formats)))
             # Structured dtype: just validate the names as needed
             else:
                 ndtype.names = validate(names, nbfields=nbtypes,
@@ -834,11 +944,10 @@ def easy_dtype(ndtype, names=None, defaultfmt="f%i", **validationargs):
         elif (nbtypes > 0):
             validate = NameValidator(**validationargs)
             # Default initial names : should we change the format ?
-            if (ndtype.names == tuple("f%i" % i for i in range(nbtypes))) and \
-               (defaultfmt != "f%i"):
+            if ((ndtype.names == tuple("f%i" % i for i in range(nbtypes))) and
+                    (defaultfmt != "f%i")):
                 ndtype.names = validate([''] * nbtypes, defaultfmt=defaultfmt)
             # Explicit initial names : just validate
             else:
                 ndtype.names = validate(ndtype.names, defaultfmt=defaultfmt)
     return ndtype
-
